// g++ -I ~/metis -o partition partition.cpp -L ~/metis -lmetis
extern "C"
{
	#include <metis.h>
}
#include <stdio.h>

#define N 44
#define NPARTS 3
#define EDGES 216

int main(void)
{
	int n = N;

	int xadj[N + 1] = {1, 4, 8, 12, 18, 24, 30, 34, 38, 44, 50, 56, 62, 69, 72, 75, 80, 86, 92, 96, 100, 106, 113, 119, 123, 127, 133, 139, 145, 151, 157, 163, 166, 169, 175, 179, 183, 189, 195, 199, 203, 207, 211, 214, 217};
	int adjncy[EDGES] = {2, 3, 4, 1, 4, 5, 8, 1, 4, 6, 7, 1, 2, 3, 5, 6, 9, 2, 4, 8, 9, 12, 13, 3, 4, 7, 9, 10, 11, 3, 6, 10, 14, 2, 5, 12, 15, 4, 5, 6, 11, 13, 16, 6, 7, 11, 14, 18, 20, 6, 9, 10, 16, 18, 22, 5, 8, 13, 15, 17, 19, 5, 9, 12, 16, 17, 21, 23, 7, 10, 20, 8, 12, 19, 9, 11, 13, 21, 22, 12, 13, 19, 23, 25, 26, 10, 11, 20, 22, 24, 27, 12, 15, 17, 25, 10, 14, 18, 24, 13, 16, 22, 23, 28, 29, 11, 16, 18, 21, 27, 28, 30, 13, 17, 21, 26, 29, 31, 18, 20, 27, 33, 17, 19, 26, 32, 17, 23, 25, 31, 32, 35, 18, 22, 24, 30, 33, 36, 21, 22, 29, 30, 34, 38, 21, 23, 28, 31, 34, 37, 22, 27, 28, 36, 38, 39, 23, 26, 29, 35, 37, 40, 25, 26, 35, 24, 27, 36, 28, 29, 37, 38, 41, 42, 26, 31, 32, 40, 27, 30, 33, 39, 29, 31, 34, 40, 41, 43, 28, 30, 34, 39, 42, 44, 30, 36, 38, 44, 31, 35, 37, 43, 34, 37, 42, 43, 34, 38, 41, 44, 37, 40, 41, 38, 39, 42};
	int wgtflag = 0; // Sin pesos
	int numflag = 1;
	int nparts = NPARTS; // Numero de particiones
	int options[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	int edgecut;
	int part[N];
	METIS_PartGraphKway(&n, xadj, adjncy, NULL, NULL, &wgtflag, &numflag, &nparts, options, &edgecut, part);

	// Imprimir particiones
	for (int p = 1; p <= NPARTS; ++p)
	{
		printf("\nPartition %i:\n", p);
		for (int i = 0; i < N; ++i)
		{
			if (part[i] == p)
			{
				printf("%i,", i + 1);
			}
		}
		printf("\n");
	}
	printf("\n");

	// Encontrar fronteras
	bool is_boundary[N] = {false};
	for (int i = 0; i < N; ++i)
	{
		int start = xadj[i] - 1;
		int size = xadj[i + 1] - xadj[i];
		int p = part[i];
		for (int j = 0; j < size; ++j)
		{
			int a = adjncy[start + j] - 1;
			int q = part[a];
			if (p < q)
			{
				is_boundary[i] = true;
				break;
			}
		}
	}

	int II[NPARTS][N];
	int IB[NPARTS][N];
	int BB[N];
	int size_II[NPARTS] = {0};
	int size_IB[NPARTS] = {0};
	int size_BB = 0;

	// Definir indices
	for (int i = 0; i < N; ++i)
	{
		if (is_boundary[i])
		{
			BB[size_BB++] = i + 1;
			int start = xadj[i] - 1;
			int size = xadj[i + 1] - xadj[i];
			for (int j = 0; j < size; ++j)
			{
				int a = adjncy[start + j] - 1;
				if (!is_boundary[a])
				{
					int p = part[a] - 1;
					if (size_IB[p] > 0)
					{
						if (IB[p][size_IB[p] - 1] == i + 1)
						{
							continue;
						}
					}
					IB[p][size_IB[p]++] = i + 1;
				}
			}
		}
		else
		{
			int p = part[i] - 1;
			II[p][size_II[p]++] = i + 1;
		}
	}

	for (int p = 0; p < NPARTS; ++p)
	{
		printf("\nParticion %i\n", p + 1);
		printf("  II:");
		for (int i = 0; i < size_II[p]; ++ i)
		{
			printf(" %i", II[p][i]);
		}
		printf("\n  IB:");
		for (int i = 0; i < size_IB[p]; ++ i)
		{
			printf(" %i", IB[p][i]);
		}
	}
	printf("\nBoundary\n");
	printf("  BB:");
	for (int i = 0; i < size_BB; ++ i)
	{
		printf(" %i", BB[i]);
	}
	printf("\n\n");

	return 0;
}
